# from scipy import ndimage as ndi
# import nibabel as nib
# 
# img1 = nib.load('my_file.nii')
# img2 = nib.load('other_file.nii.gz')
# img3 = nib.load('spm_file.img')
# 
# data = img1.get_data()
# affine = img1.affine
# 
# print(img1)
# 
# nib.save(img1, 'my_file_copy.nii.gz')
# 
# new_image = nib.Nifti1Image(data, affine)
# nib.save(new_image, 'new_image.nii.gz')

